const MODULE_NAME = 'TRANSPORT';
const SLEEP_BEFORE_TERMINATE_ON_ERROR_MS = 5 * 1000;

const bot = require('simple-xmpp');
const config = require('komodo-sdk/config');
const logger = require('tektrans-logger');
const messagingService = require('komodo-center-messaging-client-lib');
const uniqid = require('uniqid');

const customPing = require('./custom-ping');
const killOnIdle = require('./kill-on-idle');
const sender = require('./sender');

let isReady;

bot.on('online', (data) => {
    logger.info(`XMPP transport connected, JID: ${data.jid.user}`);

    sender.init(bot);

    if (config.custom_ping) {
        customPing.setBot(bot);
    }

    bot.getRoster();

    setTimeout(
        () => {
            isReady = true;

            logger.verbose('Transport is ready');

            if (messagingService.onOnline) {
                messagingService.onOnline({
                    me: config.username,
                });
            }
        },

        config.warming_up_ms || (30 * 1000),
    );
});

bot.on('chat', (partner, msg) => {
    if (!msg || !msg.trim()) {
        return;
    }

    if ((partner || '').toLowerCase() === config.username.toLowerCase().replace(/\/.*$/, '')) {
        return;
    }

    killOnIdle.touch();

    const xid = uniqid();

    if (!isReady) {
        if (!customPing.isPingMessage(msg) && !customPing.isPongMessage(msg)) {
            logger.warn('Warming up is not finished yet, ignoring message', {
                xid, me: config.username, partner, msg,
            });
        }
        return;
    }

    if (msg.toLowerCase().indexOf('error') === 0) {
        return;
    }

    if (msg.toLowerCase().indexOf('perintah salah') === 0) {
        return;
    }

    if (msg.toLowerCase().indexOf('pembelian ') === 0) {
        return;
    }

    if (customPing.isPongMessage(msg)) {
        if (config.custom_ping && config.custom_ping.verbose) {
            logger.verbose(`${MODULE_NAME} 70CDD087: Got PONG message`, {
                xid,
                partner,
                msg,
            });
        }
        return;
    }

    if (customPing.isPingMessage(msg)) {
        if (customPing.isAllowedPartner(partner)) {
            if (config.custom_ping && config.custom_ping.verbose) {
                logger.verbose(`${MODULE_NAME} ED8C8786: Processing PING message`, {
                    xid,
                    partner,
                    msg,
                });
            }

            customPing.sendPong(xid, partner, msg);
        }

        return;
    }

    logger.info('Incoming message via XMPP transport', {
        xid, me: config.username, partner, msg,
    });

    if (messagingService && messagingService.onIncomingMessage) {
        messagingService.onIncomingMessage(
            {
                me: config.username,
                partner,
                msg: msg.trim(),
                xid,
            },
        );
    }
});

bot.on('error', (err) => {
    const xid = uniqid();

    logger.warn(`${MODULE_NAME} F2E53C12: Error detected.`, {
        xid,
        eCode: err.code,
        eMessage: err.message,
    });

    if (!config.do_not_terminate_on_error) {
        logger.warn(`${MODULE_NAME} BA6C0C55: Terminating on error`, {
            xid,
            millisecondSleepBeforeTerminate: SLEEP_BEFORE_TERMINATE_ON_ERROR_MS,
        });

        setTimeout(() => {
            process.exit(1);
        }, SLEEP_BEFORE_TERMINATE_ON_ERROR_MS);
    }
});

function send(partner, msg, xid) {
    sender.send(partner, msg, xid);
}

bot.on('subscribe', (from) => {
    logger.verbose(`Incoming subscribe request from ${from}`);
    bot.acceptSubscription(from);
    bot.subscribe(from);
});

function ping() {
    const xid = uniqid();
    if (isReady) sender.send(config.username, `PING ${xid}`, xid, true);
}

function init() {
    messagingService.setTransport(exports);

    if (!killOnIdle.getDisabled()) {
        killOnIdle.init();
    }

    bot.connect({
        jid: config.username,
        password: config.password,
        host: config.xmpp_host,
    });

    setInterval(
        ping,
        config.ping_interval_ms || 25000,
    );
}

init();

exports.init = init;
exports.send = send;
